"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.getMessageForSlack = void 0;
/* eslint-disable @typescript-eslint/no-explicit-any */
const winston_slack_webhook_transport_1 = __importDefault(require("winston-slack-webhook-transport"));
const logger_types_1 = require("#/logger/logger.types");
const getMessageForSlack = (info, nodeEnv) => {
    if (typeof info.stack === 'string') {
        // info.stack = info.stack.split('\n').map((item, i) => ({ [i]: item.trim() }));
        info.stack = [
            {
                0: info.stack,
            },
        ];
    }
    if (!Array.isArray(info.stack) || info.level !== logger_types_1.LOG_LEVEL.Fatal)
        return {};
    console.log(info.stack);
    const attachments = [];
    const fields = [];
    for (const key of Object.keys(info.stack)) {
        fields.push({
            title: 'Raw Stack:',
            value: JSON.stringify(info.stack[key], null, '\t'),
            short: false,
        });
    }
    console.log(fields);
    attachments.push({
        // green
        color: '#36a64f',
        fields,
        footer: `${nodeEnv}`.toUpperCase(),
    });
    const slackMessage = {
        attachments,
        blocks: [
            {
                type: 'header',
                text: {
                    type: 'plain_text',
                    text: ':x:The service got an error :x:',
                    emoji: true,
                },
            },
            {
                type: 'context',
                elements: [
                    {
                        type: 'mrkdwn',
                        text: `:exclamation:*${nodeEnv}*`.toUpperCase() + `:exclamation:`,
                    },
                ],
            },
            {
                type: 'divider',
            },
            {
                type: 'section',
                fields: [
                    {
                        type: 'mrkdwn',
                        text: '*Error Context:*',
                    },
                    {
                        type: 'mrkdwn',
                        text: info?.context,
                    },
                ],
            },
            {
                type: 'divider',
            },
            {
                type: 'section',
                fields: [
                    {
                        type: 'mrkdwn',
                        text: '*Error Response:*',
                    },
                    {
                        type: 'mrkdwn',
                        text: info?.stack[0]?.response,
                    },
                ],
            },
            {
                type: 'divider',
            },
            {
                type: 'section',
                fields: [
                    {
                        type: 'mrkdwn',
                        text: '*Error Message:*',
                    },
                    {
                        type: 'mrkdwn',
                        text: info?.message,
                    },
                ],
            },
        ],
    };
    console.log(slackMessage);
    return slackMessage;
};
exports.getMessageForSlack = getMessageForSlack;
class SlackTransport {
    static create(webhookUrl, nodeEnv) {
        return new winston_slack_webhook_transport_1.default({
            level: logger_types_1.LOG_LEVEL.Fatal,
            webhookUrl,
            formatter: (info) => (0, exports.getMessageForSlack)(info, nodeEnv),
        });
    }
}
exports.default = SlackTransport;
