/* eslint-disable @typescript-eslint/no-explicit-any */
import SlackHook from 'winston-slack-webhook-transport';

import { LOG_LEVEL } from '#/logger/logger.types';

import { AttachementField, Attachment, ErrorMessageWithAttachment } from './slack.transport.types';

export const getMessageForSlack = (info: any, nodeEnv: string): ErrorMessageWithAttachment | object => {
	if (typeof info.stack === 'string') {
		// info.stack = info.stack.split('\n').map((item, i) => ({ [i]: item.trim() }));
		info.stack = [
			{
				0: info.stack,
			},
		];
	}

	if (!Array.isArray(info.stack) || info.level !== LOG_LEVEL.Fatal) return {};

	console.log(info.stack);
	const attachments: Attachment[] = [];
	const fields: AttachementField[] = [];

	for (const key of Object.keys(info.stack)) {
		fields.push({
			title: 'Raw Stack:',
			value: JSON.stringify(info.stack[key], null, '\t'),
			short: false,
		});
	}

	console.log(fields);

	attachments.push({
		// green
		color: '#36a64f',
		fields,
		footer: `${nodeEnv}`.toUpperCase(),
	});

	const slackMessage: ErrorMessageWithAttachment = {
		attachments,
		blocks: [
			{
				type: 'header',
				text: {
					type: 'plain_text',
					text: ':x:The service got an error :x:',
					emoji: true,
				},
			},
			{
				type: 'context',
				elements: [
					{
						type: 'mrkdwn',
						text: `:exclamation:*${nodeEnv}*`.toUpperCase() + `:exclamation:`,
					},
				],
			},
			{
				type: 'divider',
			},
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error Context:*',
					},
					{
						type: 'mrkdwn',
						text: info?.context,
					},
				],
			},
			{
				type: 'divider',
			},
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error Response:*',
					},
					{
						type: 'mrkdwn',
						text: info?.stack[0]?.response,
					},
				],
			},
			{
				type: 'divider',
			},
			{
				type: 'section',
				fields: [
					{
						type: 'mrkdwn',
						text: '*Error Message:*',
					},
					{
						type: 'mrkdwn',
						text: info?.message,
					},
				],
			},
		],
	};

	console.log(slackMessage);

	return slackMessage;
};

export default class SlackTransport {
	public static create(webhookUrl: string, nodeEnv: string) {
		return new SlackHook({
			level: LOG_LEVEL.Fatal,
			webhookUrl,
			formatter: (info: any) => getMessageForSlack(info, nodeEnv),
		});
	}
}
